-- generator sinusnega signala (NCO) s tabelo in nastavljivim korakom
-- parametri NCO: 16-bitni števec, 32 vzorcev/periodo, 8-bitni izhod

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity gen is
 port (
   clk : in std_logic;
   korak : in unsigned(15 downto 0);
   data : out signed(7 downto 0) );
end gen;

architecture RTL of gen is
 signal st : unsigned(15 downto 0) := to_unsigned(0,16);
 
 -- 1/4 sinusne funkcije z amplitudo 127
 type tab_type is array (0 to 7) of signed(7 downto 0);
 constant tab : tab_type := ("00001100", "00100101", "00111100", "01010001", "01100010", "01110000", "01111010", "01111110");

 signal adr : unsigned(2 downto 0);
 signal sin : signed(7 downto 0);
begin
 -- Stevec za dolocanje faze izhodnega signala. Signal korak za nastavitev frekvence.
 process(clk)
 begin
  if rising_edge(clk) then
   st <= st + korak;
  end if;
 end process;

 -- iz zgornjih bitov stevca izracunamo naslov sinusne tabele in upostevmo simetrijo
 adr <= st(13 downto 11) when st(14) = '0' else not st(13 downto 11);
 sin <= tab(to_integer(adr));
 data <= sin when st(15) = '0' else - sin;

end RTL;